@@ -54,9 +54,12 @@ class IOSLibrary(object):
         self._screenshot_index = 0
         self._current_orientation = 0
         self._waxsim = self._find_waxsim()
+        self._username = None
+        self._password = None
         if os.path.exists(DEFAULT_SIMULATOR):
             self.set_simulator(DEFAULT_SIMULATOR)
         self._device = "iPhone"
+        self._ios_major_version = 5
 
     def set_device_url(self, url):
         """
@@ -66,6 +69,17 @@ class IOSLibrary(object):
         """
         self._url = url
 
+    def set_basic_auth(self, username, password):
+        '''
+        Set basic authentication to use with all further API calls
+
+        username is the username to authenticate with, e.g. 'Aladdin'
+
+        password is the password to use, e.g. 'open sesame'
+        '''
+        self._username = username
+        self._password = password
+
     def _find_waxsim(self):
         path = os.environ['PATH']
         for d in path.split(os.pathsep):
@@ -87,29 +101,48 @@ class IOSLibrary(object):
         """
         self._simulator = simulator_path
 
-    def set_device(self, device_name):
+    def set_device(self, device_name="iPhone"):
         """
-        Set the simulated device
+        Set the device. This is used for the simulator as well as for choosing
+        the correct gestures.
 
-        `device` The device to simulate. Valid values are: "iPhone", "iPad", "iPhone (Retina)" and "iPad (Retina)"
+        `device` The device that is connected. Valid values are:
+        "iPhone", "iPad", "iPhone (Retina)" and "iPad (Retina)"
         """
+        allowed = ("iPhone", "iPad", "iPhone (Retina)", "iPad (Retina)")
+
+        assert device_name in allowed, "%s is not in %r, but should be." % (device_name, allowed)
+
         self._device = device_name
 
 
+    def set_ios_version(self, ios_major_version=5):
+        """
+        Set the iOS Version used for sending the correct gestures.
+
+        `ios_version` The iOS version of the device that is connceted. Valid
+        values are: 4, 5, 6, must be a number.
+        """
+
+        assert type(ios_version) is int, "%s is not a number, but should be." % (device_name, allowed)
+
+        self._ios_major_version = ios_major_version
+
     def _get_app_and_binary(self, app_path):
         filename, ext = os.path.splitext(app_path)
         binary = None
         if ext == '.app':
-            binary = os.path.join(app_path,filename)
+            binary = os.path.join(app_path, filename)
         elif ext == '':
             app_path = os.path.dirname(app_path)
             binary = filename
         return app_path, binary
 
     def _check_simulator(self):
-        assert (os.path.exists(self._simulator) or (self._waxsim and os.path.exists(self._waxsim))), (
-                "neither simulator at %s nor waxsim could be found" % self._simulator)
-
+        assert (os.path.exists(self._simulator) or
+                (self._waxsim and os.path.exists(self._waxsim))), (
+                "neither simulator at %s nor waxsim could be found"
+                % self._simulator)
 
     def start_simulator(self, app_path, sdk='5.1'):
         """
@@ -119,14 +152,16 @@ class IOSLibrary(object):
         """
         self._check_simulator()
         app_path = os.path.expanduser(app_path)
-        assert os.path.exists(app_path), "Couldn't find app bundle or binary at %s" % app_path
+        assert os.path.exists(app_path), \
+                "Couldn't find app bundle or binary at %s" % app_path
 
         cmd = []
         app_path, binary = self._get_app_and_binary(app_path)
         if not self._waxsim:
 
             assert binary, "Could not parse app binary name"
-            assert os.path.exists(binary), "Could not find app binary at %s" % app_path
+            assert os.path.exists(binary), \
+                "Could not find app binary at %s" % app_path
             logging.warning("Waxsim not found, execute app without installing it in simulator")
             cmd = [self._simulator,
                   '-SimulateDevice',
@@ -151,7 +186,9 @@ class IOSLibrary(object):
                         os.path.join(os.path.dirname(__file__), 'resources'),
                         "reset.applescript")
         cmd = ["osascript",p]
-        subprocess.Popen(cmd)
+        with open("reset_sim.log","w") as logfile:
+            with open("reset_sim.err.log","w") as errfile:
+                self._reset = subprocess.Popen(cmd, stdout=logfile, stderr=errfile)
 
     def stop_simulator(self):
         """
@@ -162,6 +199,17 @@ class IOSLibrary(object):
         stop_proc.wait()
         self._simulator_proc.wait()
 
+    def set_basic_auth(self, username, password):
+        '''
+        Set basic authentication to use with all further API calls
+
+        username is the username to authenticate with, e.g. 'Aladdin'
+
+        password is the password to use, e.g. 'open sesame'
+        '''
+        self._username = username
+        self._password = password
+
     def is_device_available(self):
         """
         Succeeds if the test server is available for receiving commands.
@@ -172,7 +220,9 @@ class IOSLibrary(object):
         Example:
         | Wait Until Keyword Succeeds | 1 minute | 10 seconds | Is device available |
         """
-        logging.getLogger().setLevel(logging.ERROR)
+        logger = logging.getLogger()
+        previous_loglevel = logger.getEffectiveLevel()
+        logger.setLevel(logging.ERROR)
         status_code = 0
         try:
             resp = self._get('version')
@@ -180,19 +230,23 @@ class IOSLibrary(object):
         except:
             raise
         finally:
-            logging.getLogger().setLevel(logging.WARNING)
+            logger.setLevel(previous_loglevel)
         assert status_code == 200, "Device is not available"
 
-    def _post(self, endp, request):
+    def _post(self, endp, request, **kwargs):
+        if self._username != None and self._username != None:
+            kwargs['auth'] = (self._username, self._password)
         url = urljoin(self._url, endp)
         res = requests.post(url, data=request, headers={
           'Content-Type': 'application/json;charset=utf-8'
-        })
+        }, **kwargs)
 
         return res
 
-    def _get(self, endp):
-        res = requests.get(urljoin(self._url, endp))
+    def _get(self, endp, **kwargs):
+        if self._username != None and self._username != None:
+            kwargs['auth'] = (self._username, self._password)
+        res = requests.get(urljoin(self._url, endp), **kwargs)
         assert res.status_code == 200, (
                 "Device sent http status code %d" % res.status_code)
         return res
@@ -209,12 +263,20 @@ class IOSLibrary(object):
         })
         res = self._post("map", data)
         logging.debug("<< %r %r", res.status_code, res.text)
-        res = json.loads(res.text)
+
+        res = self._parse_json(res.text)
+
         if res['outcome'] != 'SUCCESS':
             raise IOSLibraryException('map %s failed because: %s \n %s' %
                                       (query, res['reason'], res['details']))
         return res['results']
 
+    def _parse_json(self, to_parse):
+        try:
+            return json.loads(to_parse)
+        except ValueError as e:
+            raise IOSLibraryException("Testserver response '%s' couldn't be parsed as json: %s" % (to_parse, e.message))
+
     def _screenshot(self, filename=None):
         res = self._get('screenshot')
         path, link = self._get_screenshot_paths(filename)
@@ -223,19 +285,25 @@ class IOSLibrary(object):
         logger.info('</td></tr><tr><td colspan="3"><a href="%s">'
                    '<img src="%s"></a>' % (link, link), True, False)
 
-    def _load_playback_data(self, recording, options=None):
-        if options is None:
-            options = {}
-        ios = options.get("OS", "ios5")
-        device = options.get("DEVICE", "iphone")
+    def _load_playback_data(self, recording):
+
         if not recording.endswith(".base64"):
-            recording = "%s_%s_%s.base64" % (recording, ios, device)
+            recording = "%s_ios%d_%s.base64" % (
+                    recording,
+                    self._ios_major_version,
+                    self._device.split(" ")[0].lower())
+
         p = os.path.join(
-                        os.path.join(os.path.dirname(__file__), 'resources'),
-                        recording)
+            os.path.join(os.path.dirname(__file__), 'resources'),
+            recording
+        )
+
         if os.path.exists(p):
             with open(p, 'r') as f:
                 return f.read()
+        elif self._ios_major_version == 6:
+            # we can reuse most of the ios5 gestures on ios6
+            return self._load_playback_data(recording.replace("6", "5"))
         else:
             raise IOSLibraryException('Playback not found: %s' % p)
 
@@ -247,10 +315,21 @@ class IOSLibrary(object):
         if options:
             post_data.update(options)
         res = self._post('play', json.dumps(post_data))
-        jres = json.loads(res.text)
-        if res.status_code != 200 or jres['outcome']!='SUCCESS':
-            raise IOSLibraryException('playback failed because: %s \n %s' %
-                                       (jres['reason'], jres['details']))
+        fail = False
+        if res.status_code != 200:
+            fail = True 
+            error_msg = "device url sent status code %s", res.status_code
+
+        try:
+            jres = self._parse_json(res.text)
+            if jres['outcome'] != 'SUCCESS':
+                fail = True
+                error_msg = "%s %s" % (jres['reason'], jres['details'])
+        except ValueError:
+            pass
+
+        if fail:
+            raise IOSLibraryException('playback failed because: %s' % error_msg)
         return res
 
     def _rotate_to(self, orientation, direction="left"):
@@ -278,8 +357,12 @@ class IOSLibrary(object):
             return False
         return True
 
-    def _get_webview_html(self, index):
-        res = self.query("webView css:'body'")
+    def _get_webview_html(self, query=None, index=None):
+        if not index: index = 0
+        if not query: query = ""
+
+        res = self.query("webView " + (query and query + " " or "") + "css:'body'")
+
         index = int(index)
         if not res or not res[index]:
             raise IOSLibraryException("No WebView with index %i found" % index)
@@ -459,17 +542,24 @@ class IOSLibrary(object):
             options = {"query": query}
         self._pinch(direction, options)
 
-    def swipe(self, direction):
+    def swipe(self, direction, query=None):
         """
         Swipe.
 
         `direction` The direction to swipe in. Valid values are "up", "down", "left", "right"
+
+        `query` query identifiying the element of the screen to be swiped on, e.g. "view marked:'foo'"
         """
         degrees = ORIENTATIONS[direction]
         direction = (360 - self._current_orientation) + degrees
         direction = self._reduce_degrees(direction)
         direction = ORIENTATIONS_REV[direction]
-        self._playback("swipe_%s" % direction)
+
+        options = {}
+        if query:
+            options["query"] = query
+
+        self._playback("swipe_%s" % direction, options)
 
     def screen_should_contain_text(self, expected):
         """
@@ -477,7 +567,8 @@ class IOSLibrary(object):
 
         `expected` The text that should be on the screen
         """
-        if not self._element_exists("view {text == '%s'}" %
+
+        if not self._element_exists("view {text LIKE '*%s*'}" %
                                     expected.replace("'", r"\'")):
             raise IOSLibraryException("No text %s found" % expected)
 
@@ -494,24 +585,28 @@ class IOSLibrary(object):
             raise IOSLibraryException("No element found with mark or text %s" %
                                       expected)
 
-    def webview_should_contain(self, expected, index=0):
+    def webview_should_contain(self, expected, index=0, query=None):
         """
         Asserts that the current webview contains a given text
 
         `expected` text that should be in the webview
 
         `index` index of the webView
+
+        `query` query to find the webview (e.g. "marked:'Tears in Heaven'", for full query syntax see https://github.com/calabash/calabash-ios/wiki/05-Query-syntax
         """
-        if not expected in self._get_webview_html(index):
+        if not expected in self._get_webview_html(query, index):
             raise IOSLibraryException("%s not found in webView" % expected)
 
-    def webview_should_not_be_empty(self, index=0):
+    def webview_should_not_be_empty(self, index=0, query=None):
         """
         Asserts that the current webview is not empty
 
         `index` index of the webView
+
+        `query` query to find the webview (e.g. "marked:'Tears in Heaven'", for full query syntax see https://github.com/calabash/calabash-ios/wiki/05-Query-syntax
         """
-        if not self._get_webview_html(index):
+        if not self._get_webview_html(query, index):
             raise IOSLibraryException("Webview is empty")
 
     # END: DEFINITIONS
